# Process Mining
## Inhalt
- Process Mining: Zwischen Computational Intelligence, Data Mining und Prozessmodellierung/-analyse
  - Ziel:
    - Erkennen, Überwachen und Verbessern realer Prozesse
      - durch Extraktion von Wissen aus [Ereignislogs](#ereignislog-event-log)
  - Enthält:
    - Automatisches Erkennen von Prozessen
    - Überwachung von Abweichungen
    - Erkennen von sozialen/organisatorischen Netzwerken
    - automatische Erstellung von Simulationsmodellen
    - Erweiterung/Verbesserung von Modellen
    - Vorhersagen für Fälle
    - Ableiten von Empfehlungen basierend auf Fallhistorie

> - Process Mining ermöglicht Einsichten aus Ereignisdaten in Informationssystemen.
> - Methoden eröffnen neue Möglichkeiten zur Erkennung, Überwachung und Verbesserung von Prozessen.
> - Wachsendes Interesse an Process Mining aufgrund zunehmender Erfassung von Ereignissen und Bedarf an Prozessverbesserung.
> - Ziel des Manifests der IEEE Task Force: Erhöhung des Bekanntheitsgrads von Process Mining und Anregung für Forscher, Entwickler, Berater, Manager und Endanwender.
> - Ziel: Verbesserung der Anwendbarkeit von Process Mining als Werkzeug zur Prozessverbesserung, -überwachung und -unterstützung.

![image_272.png](image_272.png)

## Bedeutung für das BPM

![image_271.png](image_271.png)

### Verbindung zwischen Data Mining & BPM/Analyse:

- Brücke zwischen Data Mining und Geschäftsprozessmodellierung/Analyse
- Ermöglicht Einblicke in Geschäftsprozesse

### Modewörter im Bereich Business Intelligence (BI):

- Einführung von Modewörtern unter dem Slogan Business Intelligence (BI)
- Vermarktung von Berichts- und Dashboard-Werkzeugen

### Techniken zur Echtzeitüberwachung und -verbesserung von BPM

- Business Activity Monitoring (BAM) für Echtzeitüberwachung
- Complex Event Processing (CEP) zur Verarbeitung von Ereignissen

### Corporate Performance Management (CPM) und verwandte Managementansätze:

- Messung der Leistung von Prozessen und Organisationen
- Verbesserung durch *Continuous Process Improvement (CPI)*, *Business Process Improvement (BPI)*, *Total Quality Management (TQM)* und *Six Sigma*

### Nutzen von Process Mining für Managementansätze und Compliance Management:

- Unterstützung von CPM, BPI, TQM, Six Sigma
- Compliance Management, z.B. *Sarbanes-Oxley-Act (SOX)* und *Basel-II-Vorschriften*

### Erfassung und Analyse von Ereignisdaten:

- Ereignisdaten jederzeit verfügbar
- Weiterentwicklung von Process-Mining-Techniken

### Interesse von Wissenschaft und Industrie an Process Mining:

- Aktives Interesse an Process Mining in Wissenschaft und Industrie
- "Heißes Thema" im Geschäftsprozessmanagement

### Softwareprodukte mit Process-Mining-Funktionalität:

- Zahlreiche Softwareprodukte bieten Funktionen für Process Mining
- Beispiele: *ARIS Process Performance Manager*, *Comprehend*, *Discovery Analyst*

### Gründung einer Task Force zum Thema Process Mining:

- Interesse an logbasierter Prozessanalyse führte zur Gründung einer Task Force
- Widmet sich dem Thema Process Mining

## Bedeutung für das BPM und Stand der Forschung

### Wachstum der digitalen Welt und Verflechtung mit Prozessen:

- **Moores Gesetz und digitales Wachstum**
  - Vorhersage von Gordon Moore über das exponentielle Wachstum digitaler Technologien.
- **Verbindung von digitaler und realer Welt**
  - Erfassung und Analyse von Ereignisdaten in Organisationen durch digitale Integration.

### Ausgangspunkt und Herausforderungen für Process Mining:

- **Nutzung von Ereignislogdaten**
  - Ziel: Einsichten und Verbesserungen in Prozessen.
- **Schwierigkeiten bei der Datenverarbeitung**
  - Herausforderungen bei der Aufbereitung von Ereignisdaten für nutzbare Einsichten.

### Methoden des Process Mining:

- **Drei Arten von Process Mining**
  - Erkennung, Übereinstimmungsprüfung, Erweiterung.
  - ![image_276.png](image_276.png)
- **Perspektiven der Datenanalyse**
  - Kontrollfluss, Organisation, Fall, Zeit.

### Falsche Annahmen und Einordnung von Process Mining:

- **Missverständnisse über Anwendungsmöglichkeiten**
  - Klärung falscher Vorstellungen zur Anwendbarkeit von Process Mining.
- **Einordnung im BPM-Lebenszyklus**
  - Integration von Process Mining in den gesamten BPM-Lebenszyklus.
  - ![image_275.png](image_275.png)

### Anwendungsbereiche und Potenziale von Process Mining:

- **Einsatz in verschiedenen Phasen des BPM-Lebenszyklus**
  - Modellierung, Analyse, Implementierung, Überwachung.
- **Beitrag zur Prozessdiagnose und -optimierung**
  - Unterstützung bei Diagnose, Anpassung, und kontinuierlicher Verbesserung.

## Eigenschaften von Process Mining

### Process Mining ist nicht auf die Kontrollflusserkennung beschränkt:
- Erkennung von Prozessmodellen basierend auf Ereignislogdaten
- Rahmen nicht auf Kontrollflussperspektive beschränkt, sondern umfasst auch Organisations-, Fall- und Zeitperspektive

### Process Mining ist nicht bloß eine spezielle Art des Data Mining:
- Bindeglied zwischen Data Mining und modellgetriebenem Geschäftsprozessmanagement
- Prozessmodelle erfordern neue Repräsentationen und Algorithmen im Vergleich zu herkömmlichen Data-Mining-Methoden

### Process Mining ist nicht auf Ex-Post-Analysen begrenzt:
- Extraktion von Wissen aus historischen Ereignisdaten
- Ergebnisse können auf laufende Fälle angewandt werden, z.B. Vorhersage der Endzeit einer Bestellung

![image_277.png](image_277.png)

## Phasen

### Planung und Einordnung (Phase 0):

- **Planung des Projekts**:
  - Bestimmung von Zielen und Vorgehensweisen.
- **Einordnung des Plans**:
  - Klärung der Rahmenbedingungen und Zielsetzungen.

### Extraktion und Vorbereitung von Daten und Modellen (Phase 1):

- **Extraktion von Daten und Modellen**:
  - Erfassung und Aufbereitung von Ereignisdaten, Modellen und Analysefragen.
- **Verständnis der verfügbaren Daten und Domäne**:
  - Klärung, welche Daten analysiert werden können und welche Fragestellungen wichtig sind.

### Ableitung des Kontrollflussmodells und Verbindung mit Ereignisdaten (Phase 2):

- **Ableitung des Kontrollflussmodells**:
  - Erstellung des Modells basierend auf Ereignisdaten.
- **Verbindung von Modell und Ereignisdaten**:
  - Anpassung und Ergänzung des Modells mit Hilfe von Ereignisdaten.

### Ergänzung um weitere Perspektiven und Erweiterung des Modells (Phase 3):

- **Ergänzung um weitere Perspektiven**:
  - Einbeziehung zusätzlicher Informationen wie Daten, Zeit und Ressourcen.
- **Erweiterung des Modells**:
  - Anpassung und Erweiterung des Modells basierend auf zusätzlichen Informationen und Perspektiven.

### Operative Unterstützung und Nutzung des Modells (Phase 4):

- **Operative Nutzung des Modells**:
  - Einsatz des Modells zur operativen Unterstützung und Entscheidungsfindung.
- **Zusammenführung von historischen und aktuellen Daten**:
  - Integration von historischen Daten und aktuellen Informationen zur Interventionsmöglichkeit und Vorhersage.

### Herausforderungen und Potenziale von Process Mining:

- **Unausgereifte Werkzeuge und mangelnde Vertrautheit**:
  - Herausforderungen bei der Nutzung von Process-Mining-Werkzeugen aufgrund ihrer Unaushgereiftheit und mangelnder Vertrautheit der Nutzer.
- **Leitsätze zur Weiterentwicklung**:
  - Benennung von Leitsätzen im Manifest, um Anwender, Forscher und Entwickler bei der Weiterentwicklung von Methoden zu unterstützen.

## Leitsätze für Process-Mining
![image_279.png](image_279.png)
### LS1: Qualität der Ereignisdaten sicherstellen

- **Ereignisse als fundamentale Informationsträger**:
  - Ereignisse sind Ausgangspunkt für alle Process-Mining-Aktivitäten.
- **Ereignislogs von hoher Qualität**:
  - Belastbare, vollständige Ereignisdaten mit definierter Semantik und sicherem Datenschutz sind essentiell.
- **Reifegrade von Ereignislogs**:
  - Unterschiedliche Qualitätsgrade von Ereignislogs beeinflussen die Anwendbarkeit von Process Mining.
  - ![image_278.png](image_278.png)

### LS2: Konkrete Fragestellungen treiben Datenextraktion

- **Fragestellungen als Treiber**:
  - Konkrete Fragestellungen sind entscheidend für die Auswahl relevanter Ereignisdaten.
- **Bestimmung des Falltyps**:
  - Auswahl des Prozessmodells basierend auf den zu beantwortenden Fragen und Falltypen.

### LS3: Unterstützung elementarer Kontrollflusskonzepte

- **Unterstützung von Kontrollflussmustern**:
  - Process-Mining-Techniken sollten grundlegende Workflow-Konzepte wie Sequenzen, Parallelität und Entscheidungen unterstützen.
- **Bedeutung der Parallelität**:
  - Fehlende Unterstützung von Parallelität kann zu ungenauen Modellen führen.

### LS4: Beziehung zwischen Ereignissen und Modellen herstellen

- **Bedeutung der Beziehung**:
  - Die Beziehung zwischen Ereignissen und Modellen ist entscheidend für Analyse und Modellierung.
- **Eindeutige Zuordnung von Ereignissen**:
  - Klare Zuordnung von Ereignissen zu Aktivitäten und Prozessinstanzen ist notwendig für eine korrekte Interpretation.

### LS5: Modelle als zweckmäßige Abstraktionen der Realität

- **Modelle als Sichten auf die Realität**:
  - Prozessmodelle sollten zweckmäßige Abstraktionen darstellen, die den Bedürfnissen verschiedener Akteure gerecht werden.
- **Variabilität der Modelle**:
  - Unterschiedliche Perspektiven, Granularitäten und Darstellungsformen sind je nach Einsatzzweck erforderlich.

### LS6: Kontinuierlicher Prozess und aktive Nutzung von Modellen

- **Kontinuierlicher Prozess des Process Mining**:
  - Process Mining sollte als kontinuierlicher Prozess betrachtet werden, der laufend aktualisierte und entscheidungsrelevante Informationen bereitstellt.
- **Aktive Nutzung von Modellen**:
  - Modelle sollten regelmäßig betrachtet und mit aktuellen Ereignisdaten aktualisiert werden, um Entscheidungen zu unterstützen und Potenziale voll auszuschöpfen.


## IEEE Task Force

### Einführung

Die IEEE Task Force on Process Mining verfolgt das Ziel, die Anwendung und Forschung im Bereich Process Mining zu fördern und zu unterstützen. Dieses Manifest beschreibt die Grundprinzipien und Intentionen der Task Force sowie einige Leitsätze und Herausforderungen.

### Leitsätze

1. **Förderung und Verbreitung von Process Mining**: Die Task Force strebt danach, die Anwendung von Process Mining zu fördern und zu verbreiten, indem sie Softwareentwickler, Berater, Manager und Endnutzer bei der Nutzung modernster Techniken unterstützt.

2. **Unterstützung der Forschung**: Ein weiteres Ziel ist die Unterstützung und Förderung der Forschung im Bereich Process Mining, um Innovationen voranzutreiben und neue Erkenntnisse zu gewinnen.

3. **Vermeidung häufiger Fehler**: Durch die Bereitstellung von Leitsätzen und Herausforderungen soll dazu beigetragen werden, häufige Fehler zu verhindern und den Reifegrad von Process-Mining-Techniken zu erhöhen.

### Herausforderungen

- **Terminologie**: Die Verwendung unterschiedlicher Begriffe und Terminologien in verschiedenen Organisationen kann Verwirrung stiften. Die Task Force strebt an, Klarheit und Einheitlichkeit in der Terminologie zu schaffen, indem der Begriff "Process Mining" für die beschriebene Disziplin verwendet wird.

### Terminologie

- **Workflow Mining**: Beschreibt die Erstellung von Workflow-Modellen als eine von vielen Anwendungen von Process Mining.

- **(Business) Process Mining**: Eine breitere Bezeichnung, die auf die Anwendung von Process Mining-Techniken in geschäftlichen Kontexten hinweist.

- **Automatische (Geschäfts-) Prozesserkennung**: Eine spezifische Art von Process Mining, die darauf abzielt, automatisch Prozessmodelle aus Ereignisdaten zu extrahieren.

- **(Business) Process Intelligence**: Eine Kombination von Business Intelligence (BI) und Business Process Management (BPM), die BI-Techniken zur Verbesserung von Prozessen nutzt.

Die Task Force empfiehlt die Verwendung des Begriffs "Process Mining" für die beschriebene Disziplin und betont die Bedeutung von Klarheit und Einheitlichkeit in der Terminologie.


## Begriffe
### Zusammenhänge
![image_280.png](image_280.png)

### Erklärungen
#### Aktivität (Activity):
> - Wohl-definierter Schritt im Prozess
> - Ereignisverweis auf Start, Abschluss, etc.


#### Business Intelligence (BI):
> - Werkzeuge und Methoden für Entscheidungsunterstützung


#### Darstellungsausrichtung:
> - Gewählte Zielsprache für Ergebnispräsentation

#### Data Mining:
> - Analyse großer Datenmengen für neue Erkenntnisse

#### Einfachheit (Simplicity):
> - Occams Rasiermesser: einfachstes erklärbares Modell

#### Ereignis (Event):
> - Aufgezeichnete Aktion im Log

#### Ereignislog (Event Log):
> - Sammlung von Ereignissen für Process Mining

#### Erweiterung (Model Enhancement):
> - Verbesserung des Prozessmodells durch Log-Informationen
