# Prozesscontrolling und Prozesssimulation
![image_383.png](image_383.png)
## Lernziele
1. [Kennzahlen für Controlling](#eingangsparameter)
2. [Durchlaufzeit/Zykluszeit analytisch berechnen](#berechnung-der-dlz)
3. Elemente einer [Prozesskostenrechnung](#eingangsparameter) beschreiben
4. einfache [Prozesskostenrechnung](#prozesskostenrechnung-pkr) durchführen
5. [Little's Law](#little-s-law) erläutern, anwenden
6. Grundprinzipien/allgemeines Vorgehen von Simulationen
7. Vor-/Nachteile von Simulationen

## Prozesscontrolling
## Analytische Verfahren zur Untersuchung von Prozessen
### Statische Analyse
| Qualitative Analyse                           | Quantitative Analyse                                                     |
|-----------------------------------------------|--------------------------------------------------------------------------|
| Schwerpunkt auf Konsistenz, Gültigkeit der PM | Mathematische Bewertung                                                  |
|                                               | Verarbeitungszeit/-kosten einer einzelnen Aktivität                      |
|                                               | ca. Auftretenswahrscheinlichkeit                                         |
|                                               | Minimale, Maximale, Durchschnitt Zeit/Kosten einer Folge von Aktivitäten |
|                                               | Erforderliches Personal                                                  |

#### Qualitative statische Analyse (Schwerpunkt Konsistenz, Validität)
- Überprüfung Modellqualität
    - Skript erstellt Prozessbericht
- **Beispiel**
    - Alle Aktivitäten melden, die
        - Bearbeitungszeit > / < x
        - erforderlich für Bearbeitung von Formular
        - manuellen Eingriff brauchen

#### Verbesserungsmöglichkeiten identifiziert
- Prozesstätigkeiten ohne klare Zuständigkeit
- zeit-/kostenintensiven Aktivitäten (_Kostentreiber_)
- Gemeinsame Ressourcennutzung (potenzielle Engpässe)
- Zusammenlegung / Integration von Aktivitäten


### Quantitative Analyse
- **Ziel**
    - Berechnung Kosten/Zeit pro Prozess
        - Minimal, Durchschnittlich, Maximal
- **Erforderliche Informationen**
    - Zeit- & Kosteninformationen
    - Häufigkeit

## Eingangsparameter
### Zeit & Kosten
#### Zeit
- Rüstzeit
- Liegezeit
- Bearbeitungszeit
- Transportzeit

#### Kosten
- Personal
- Material
- Gemeinkosten
    - Verwaltung
    - indirekte Kosten

### Häufigkeiten und Verteilungen
#### Häufigkeiten
- Wie oft in einer best. Zeitspanne

#### Verteilungen
- Entscheidungspunkte im Prozess
    - bspw. 40%/60% Pfad A/B

## Zeitorientierte Größen
### Durchlaufzeit (DLZ)
- Zeitdauer, die Objekt im Prozess benötigt
- Beinhaltet:
    - **Bearbeitungs-/Prozesszeit (BZ/PZ)**
    - **Wartezeit (WZ)**
    - **Liegezeit (LZ)**
        - Zeit, welche Aktivitäten durchschnittlich nach Bearbeitung bei Akteuren liegen
    - **Transportzeit (TZ)**
        - Zeit, die durchschnittlich durch Transport zwischen Aktivitäten anfällt
- **Aktivitätszeit** = BZ + WZ + LZ + TZ
- **Zykluszeit**
    - Gesamtsumme aller Durchlaufzeiten aller Prozesspfade
    - ![image_385.png](image_385.png)

#### Durchschnittliche Durchlaufzeit
- komprimierte Zeitdauer eines Prozesses
- ![image_384.png](image_384.png)

### Zeiteffizienz
- Ermittlung des Zeitniveaus
    - \> 10% → gut
    - < 5% → Verbesserungspotential
- = BZ/DLZ

### Termintreue
- Zuverlässigkeit des Prozesses
- ![image_386.png](image_386.png)

## Berechnung der [DLZ](#durchlaufzeit-dlz)
### Sequenzielle Aktivitäten
- ![image_387.png](image_387.png)
- Addition

### Parallele Gateways
- ![image_388.png](image_388.png)
- Maximalwert aller Werte (Zeit)
- Summe (Kosten)
- ![image_390.png](image_390.png)

### Exklusive Gateways
- ![image_389.png](image_389.png)
- |        | Minimum               | Durchschnitt                          | Maximum               |
  |--------|-----------------------|---------------------------------------|-----------------------|
  | Zeit   | min(aller Werte) 2    | Summe(Wahrscheinlichkeit*Zeit) 2,7    | Max(aller Werte) 3    |
  | Kosten | min(aller Werte) 1200 | Summe(Wahrscheinlichkeit*Kosten) 1760 | Max(aller Werte) 2000 |

### Inklusives Gateway
- ![image_391.png](image_391.png)
- |        | Minimum               | Durchschnitt                               | Maximum               |
  |--------|-----------------------|--------------------------------------------|-----------------------|
  | Zeit   | Min(aller Werte) 2    | Summe(Prob. * Zeit)/Summe(alle prob.) 2,75 | Max(aller Werte) 3    |
  | Kosten | Min(aller Werte) 1200 | Summe(prob * Kosten) 2160                  | Max(aller Werte) 3200 |


### Nacharbeiten (Schleifen)
- ![image_392.png](image_392.png)
- | einfacher Rework | mehrfacher Rework |
  |------------------|-------------------|
  | (1+r) * T        | T / (1-r)         |
  | 54min            | 56,25min          |
  - T = Summe der Aktivitätszeiten in der Schleife
  - r = Wahrscheinlichkeit für rework

### Übung
- ![image_393.png](image_393.png)

## Prozesskostenrechnung (PKR)
- Abrechnungstechnik
  - ordnet Kosten den einzelnen Tätigkeiten zu
- Wählt Mengengerüst der durch einen GP laufenden Transaktionen
  - Kosten auf Aktivitäten/GP verrechnen
- Einschränkungen
  - Kosten werden nicht verändert/gelöscht
  - Liefert nur **Informationen wo welche Kosten entstehen**
    - **Tätigkeiten und Kosten im Vergleich** (_vor/nach Prozessgestaltung_)
    - **Häufigkeit und Kosten der Tätigkeiten** (_vor/nach Prozessgestaltung_)
    - **Kosten der Null-Variante**
      - Was, wenn Prozessgestaltung nicht umgesetzt wird
    - Welche Prozesse(/-teile) sind (wie) **wertschöpfend**

### Grundsätze PKR
- Aktivitäten verbrauchen Ressourcen
  - verursacht Kosten
- Umwandlung von indirekten in direkte Kosten

### Anwendungszweck PKR
- Hohe Gemeinkosten (bspw. Bereich Fertigung und Entwicklung)
- hohe Fehlerkosten
- Ineffizienz
- Ausgeprägter Konkurrenz

### Beispiel PKR
![image_394.png](image_394.png)

## Little's Law
- 3 Kennzahlen für Prozesse im Verhältnis
  - [Durchlaufzeit](#durchlaufzeit-dlz)
  - **Output-Rate (_Flow Rate/Throughput_)**
    - Menge des Outputs pro Zeiteinheit
  - **Bestand (Inventory)**
    - Anzahl der (Durchfluss-)Einheiten
    - **Bestand = Output-Rate * Durchlaufzeit**
- **Durchflusseinheit**
  - Art des Produkts oder der Dienstleistung, mit der der Prozess zu tun hat

### Beispiele Little's Law
|                   | US Immigration            | Champagne Industry          | MBA Program                 | Large Car Manufactor |
|-------------------|---------------------------|-----------------------------|-----------------------------|----------------------|
| Durchflusseinheit | Antrag                    | Flasche Champagner          | Student                     | Auto                 |
| Output-Rate       | Bearbeitete Fälle pro Tag | Verkaufte Flaschen pro Jahr | Abschlussklasse             | Verkäufe pro Jahr    |
| Durchlaufzeit     | Bearbeitungszeit          | Zeit im Weinkeller          | 2 Jahre                     | 60 Tage              |
| Bestand           | Offene Fälle              | Inhalt Weinkeller           | Anzahl Studenten des Campus | Inventar             |

### Durchführung Little's Law
1. Beobachten des Patienteninventars zu zufälligen Zeitpunkten
   - Durchschnittswert erhalten
2. Behandlungsscheine/Aufzeichnungen zählen
   - Output-Rate erhalten
3. DLZ berechnen

#### Beispiel Durchführung Little's Law
![image_395.png](image_395.png)

### Bemerkungen / Annahmen Little's Law
- Zufluss und Abfluss sind langfristig ausgeglichen
- Robust gegenüber Schwankungen
- Alles Durchschnittswerte
  - Schwankungen drumherum werden ignoriert
- Hängt nicht von Reihenfolge ab in der Flusseinheiten bedient werden
- Was innerhalb der Blackbox passiert spielt keine Rolle

### Übung Little's Law
![image_396.png](image_396.png)

## Prozesssimulation
### Definition
- Nachbildung der Ist-/Soll-Realität
  - zum Optimieren
  - dynamisch experimentieren

### Nutzenpotential
- Betriebsabläufe ohne Unterbrechungen verbessern
- Vermeidung langwieriger Feinabstimmungen
- Reduzieren der DLZ, LZ
- Lokalisierung von Schwachstellen und Engpässen
- Bewertung alternativer Konzepte

### Simulation als Entscheidungsgrundlage
- ![image_439.png](image_439.png)

### Anwendungsbereiche
| Bereich                                | Beispiele für Einsatz                                                      |
|----------------------------------------|----------------------------------------------------------------------------|
| Konstruktion und CAD                   | Bewegungssimulation<br/>Montagemöglichkeiten                               |
| Fertigung und Logistik                 | Kapazitätsdimensionen neuer Maschinen<br/>Materialdurchflussuntersuchungen |
| Organisationsgestaltung der Verwaltung | Personalkapazitätsplanungen<br/>Optimierung Arbeitsabläufe                 |
| Schulungen und Training                | Ausbildung und Training neuer Mitarbeiter<br/>Unternehmensplanspiele       |

### Ziele
- ![image_440.png](image_440.png)

### Voraussetzungen
- **Vollständigkeit der Modellierung**
  - Erfassung von Zeit und Kosten
  - Bearbeiterzuordnung
  - Subprozesse
- **Angaben über Häufigkeit von Prozessausführungen**
  - Prozesskalender
  - Bearbeiterkalender
  - Prozessmengen
- **Auswertbarkeit von Entscheidungen**
  - Variablenbelegung bzw. Attributwerte
  - Übergangsbedingungen und Wahrscheinlichkeiten

### Analysegrößen
![image_441.png](image_441.png)

### Vorgehen
![image_442.png](image_442.png)

### Implementierung Simulationsmodell
- Aktivitätssicht des Workflow-Diagramms grafisch nachbilden
  - nicht Organisationsschicht
  - nicht Informationsschicht

## What-If-Analyse
- Verbesserungsinstrument
  - Bewertung von Änderungen auf Unternehmen
    - strategische, taktische, operative 
    - anhand verschiedener Szenarien
      - realitätsnah ohne Unterbrechung
- Erlaubt Beantwortung folgender Fragen
  - Wie verkürzt sich BZ wenn Ressourcen verdoppelt werden
  - Wie hoch ist Kosten-Nutzen-Verhältnis bei verkürzung der BZ
  - Wie wirkt sich Veränderung der Arbeitsschichtkonfiguration auf Betriebskosten aus

### Vor-/Nachteile

| Vorteile                                                   | Nachteile                                                       |
|------------------------------------------------------------|-----------------------------------------------------------------|
| Quantitative Auswertung von komplexen PM                   | Qualität der Ergebnisse abhängig von Qualität der Eingangsdaten |
| Prüfung von Handlungsalternativen ohne viel Risiko/Aufwand | Definition von Störgrößen problematisch                         |
| What-If Szenarien                                          | Notwendigkeit der Validierung der Plausibilität                 |
| Verbesserung Prozessbeherrschung                           | Realitätsnähe                                                   |
| Simultane Auswertung von Informations-/Materialflüssen     | Isoliertes, in sich geschlossenes System                        |
| Identifikation Schwachstellen                              | erzeugt nicht automatisch entscheidungsreife Vorschläge         |
| Auf-/Umbau nicht Notwendig                                 |                                                                 |


### Simulationswerkzeuge
- Nur mit Computer möglich

## Ebenen der Prozesssimulation (Bizagi)
![image_443.png](image_443.png)

### Ebene 1 - Prozessvalidierung
- Prozess durchläuft alle Sequenzflüsse und verhält sich wie erwartet
  - Ressourcen, BZ und Kosten nicht berücksichtigt
- Gateways synchron
- Nachrichten synchron
- Entscheidungswahrscheinlichkeiten korrekt
- Routing wie erwartet
- Alle Token beendet

### Ebene 2 - Zeitanalyse
- End-to-End-Prozesszeit messen
  - Ressourcen nicht berücksichtigen
- Best-Case-Szenario unter gegebenen Fluss-/Bearbeitungszeiten

### Ebene 3 - Kalenderanalyse
- Auswirkungen der Ressourcenverfügbarkeit im Laufe der Zeit
- Veränderte Bedingungen
  - Feiertagen, Wochenenden, Schichten, Pausen
- Zeigt
  - Unter-/Überauslastung
  - Gesamtkosten der Ressourcen
  - Gesamtkosten Aktivität
  - Verzögerungen
  - erwartete Zykluszeit


## Beispiel Prozesssimulation
![image_444.png](image_444.png)

- Ergebnis
  - ![image_445.png](image_445.png)

### Konfiguration Level 1
- ![image_446.png](image_446.png)
- Ergebnis
  - ![image_447.png](image_447.png)

### Konfiguration Level 2
- ![image_448.png](image_448.png)
- Ergebnis
  - ![image_449.png](image_449.png)

## Zusammenfassung Simulation
- Methode zur Bewertung der Leistung eines GM
  - verschiedene Aspekte
    - Zeit, Verfügbarkeit, Kosten
- Ziele:
  - Überprüfung der Prozessfähigkeit
  - Validierung der Realitätsnähe der Prozessmodelle
  - Bewertung alternative PM
- What-If-Analyse nützlich zur Bewertung der Auswirkung auf Unternehmen
  - strategische, taktische, betriebliche Änderung